from .types import (
    SemVersion
)
from .base import (
    List,
    AnyStr,
    define,
    field,
    BaseAttckModel,
    RELATIONSHIP_MAP,
    BASE_OBJECTS
)
from .actor import Actor
from .datacomponent import DataComponent
from .datasource import DataSource
from .definition import MarkingDefintion
from .identity import Identity
from .malware import Malware
from .matrix import Matrix
from .mitigation import Mitigation
from .relationship import Relationship
from .tactic import Tactic
from .technique import Technique
from .tool import Tool


TYPE_MAP = {
    'intrusion-set': Actor,
    'x-mitre-data-component': DataComponent,
    'x-mitre-data-source': DataSource,
    'marking-defintion': MarkingDefintion,
    'identity': Identity,
    'malware': Malware,
    'x-mitre-matrix': Matrix,
    'course-of-action': Mitigation,
    'relationship': Relationship,
    'x-mitre-tactic': Tactic,
    'attack-pattern': Technique,
    'tool': Tool
}


@define
class MitreAttck(BaseAttckModel):
    type: AnyStr = field()
    spec_version: SemVersion = field()
    objects: List = field()
    relationship_map: dict = field(factory=dict)

    def __attrs_post_init__(self):
        if self.objects:
            return_list = []
            for item in self.objects:
                try:
                    data = TYPE_MAP.get(item['type'])(**item)
                    return_list.append(data)
                    BASE_OBJECTS.append(data)
                except Exception as e:
                    continue
                if item['type'] == 'relationship' and item['relationship_type'] != 'revoked-by':
                    source_id = item['source_ref']
                    target_id = item['target_ref']
                    if source_id not in RELATIONSHIP_MAP:
                        RELATIONSHIP_MAP[source_id] = []
                    if target_id not in RELATIONSHIP_MAP[source_id]:
                        RELATIONSHIP_MAP[source_id].append(target_id)
                    if target_id not in RELATIONSHIP_MAP:
                        RELATIONSHIP_MAP[target_id] = []
                    if source_id not in RELATIONSHIP_MAP[target_id]:
                        RELATIONSHIP_MAP[target_id].append(source_id)
            self.objects = return_list



from datetime import datetime
from typing import (
    Any,
    AnyStr,
    List
)
from pydantic import (
    HttpUrl
)
from attr import (
    define,
    field,
    validators
)
from .types import (
    PATTERNS,
    Id,
    SemVersion
)


BASE_OBJECTS = []
RELATIONSHIP_MAP = {}


@define
class ExternalReferences:
    source_name: AnyStr = field(default=None)
    url: HttpUrl = field(default=None)
    external_id: AnyStr = field(default=None)
    description: AnyStr = field(default=None)
    source_name: AnyStr = field(default=None)

    @external_id.validator
    def validate_external_id(self, attribute, value):
        if value[0] not in ['T', 'G'] and value != 'enterprise-attack':
            raise ValueError("External ID must start with a known letter.")


@define
class BaseAttckModel:
    id: Id = field()


@define(eq=False)
class BaseModel(BaseAttckModel):
    id: Id = field()
    name: AnyStr = field()
    created: datetime = field()
    external_references: List[ExternalReferences] = field()
    modified: datetime = field()
    x_mitre_version: SemVersion = field()
    object_marking_refs: List[Id] = field()
    x_mitre_domains: List = field()

    def _get_relationship_objects(self, parent_id: str, parent_type: str) -> list:
        return_list = []
        if RELATIONSHIP_MAP.get(parent_id):
            for item in RELATIONSHIP_MAP[parent_id]:
                for x in BASE_OBJECTS:
                    if x.id == item and x.type == parent_type and x not in return_list:
                        return_list.append(x)
        return return_list

    def __attrs_post_init__(self):
        if self.external_references:
            return_list = []
            for item in self.external_references:
                return_list.append(ExternalReferences(**item))
            self.external_references = return_list



from .types import (
    Id,
    SemVersion,
    MitreDomain,
    MitrePlatform
)
from .base import (
    BaseModel,
    List,
    AnyStr,
    define,
    field,
    validators
)

@define
class Command:
    source: AnyStr = field(factory=str)
    command: AnyStr = field(factory=str)
    name: AnyStr = field(factory=str)


@define
class KillChainPhases:
    kill_chain_name: AnyStr = field()
    phase_name: AnyStr = field()


@define
class Technique(BaseModel):
    type: AnyStr = field(validator=validators.in_(['attack-pattern']))
    description: AnyStr = field()
    created_by_ref: Id = field()
    x_mitre_modified_by_ref: Id = field()
    kill_chain_phases: List[KillChainPhases] = field()
    x_mitre_platforms: List[MitrePlatform] = field()
    x_mitre_domains: List[MitreDomain] = field()
    x_mitre_data_sources: List = field()
    x_mitre_contributors: List = field()

    x_mitre_impact_type: List = field(factory=list)
    x_mitre_deprecated: bool = field(factory=bool)
    x_mitre_effective_permissions: List = field(factory=list)
    x_mitre_remote_support: bool = field(factory=bool)
    x_mitre_permissions_required: List = field(factory=list)
    x_mitre_is_subtechnique: bool = field(factory=bool)
    x_mitre_detection: AnyStr = field(factory=str)
    x_mitre_defense_bypassed: List = field(factory=list)
    x_mitre_system_requirements: List = field(factory=list)
    x_mitre_attack_spec_version: SemVersion = field(factory=SemVersion)
    revoked: bool = field(factory=bool)

    command_list: List = field(factory=list)
    commands: List[Command] = field(factory=list) # need to define this object better
    queries: List = field(factory=list) # need to define this object better
    datasets: List = field(factory=list) # need to define this object better
    possible_detections: List = field(factory=list) # need to define this object better
    external_reference: List = field(factory=list)

    controls: List = field(factory=list)

    @property
    def actors(self):
        return self._get_relationship_objects(
            parent_id=self.id,
            parent_type='intrusion-set'
        )

    @property
    def data_components(self):
        return self._get_relationship_objects(
            parent_id=self.id,
            parent_type='x-mitre-data-component'
        )

    @property
    def data_sources(self):
        return self._get_relationship_objects(
            parent_id=self.id,
            parent_type='x-mitre-data-source'
        )

    @property
    def malwares(self):
        return self._get_relationship_objects(
            parent_id=self.id,
            parent_type='malware'
        )

    @property
    def mitigations(self):
        return self._get_relationship_objects(
            parent_id=self.id,
            parent_type='course-of-action'
        )

    @property
    def tactics(self):
        return self._get_relationship_objects(
            parent_id=self.id,
            parent_type='x-mitre-tactic'
        )
    
    @property
    def techniques(self):
        return self._get_relationship_objects(
            parent_id=self.id,
            parent_type='attack-pattern'
        )

    @property
    def tools(self):
        return self._get_relationship_objects(
            parent_id=self.id,
            parent_type='tool'
        )

    def __attrs_post_init__(self):
        if self.controls:
            from .control import Control
            return_list = []
            for item in self.controls:
                try:
                    return_list.append(Control(**item))
                except ValueError as ve:
                    raise ve
            self.controls = return_list
        



from datetime import datetime
from attr import validators
from .types import (
    Id,
    SemVersion, 
    PATTERNS
)
from .base import (
    ExternalReferences,
    List,
    AnyStr,
    define,
    field
)


@define
class Statement:
    statement: AnyStr = field()


@define
class MarkingDefintion:
    type: AnyStr = field(validator=validators.in_(['marking-defintion']))
    id: Id = field()
    type: AnyStr = field(validator=validators.in_(PATTERNS['types']['examples']))
    created: datetime = field()
    definition: Statement = field()

    definition_type: AnyStr = field()
    
    created_by_ref: Id = field(factory=Id)
    x_mitre_attack_spec_version: SemVersion = field(factory=SemVersion)
    object_marking_refs: List = field(factory=list)
    external_references: List[ExternalReferences] = field(factory=list)


from .types import (
    Id,
    SemVersion,
    MitreDomain
)
from .base import (
    BaseModel,
    List,
    AnyStr,
    define,
    field,
    validators
)


@define
class Actor(BaseModel):
    type:                        AnyStr            = field(validator=validators.in_(['intrusion-set']))
    aliases:                     List              = field()
    x_mitre_contributors:        List              = field(factory=list)
    revoked:                     bool              = field(factory=bool)
    description:                 AnyStr            = field(factory=str)
    x_mitre_modified_by_ref:     Id                = field(factory=Id) 
    x_mitre_deprecated:          bool              = field(factory=bool)
    x_mitre_attack_spec_version: SemVersion        = field(factory=SemVersion)
    created_by_ref:              Id                = field(factory=Id)
    x_mitre_domains:             List[MitreDomain] = field(factory=list)

    # These additional properties are from external data sets
    country:                     List              = field(factory=list)
    operations:                  List              = field(factory=list)
    attribution_links:           List              = field(factory=list)
    known_tools:                 List              = field(factory=list)
    targets:                     List              = field(factory=list)
    additional_comments:         List              = field(factory=list)
    external_description:        List              = field(factory=list)

    @property
    def malwares(self):
        """
        Returns all malware objects that are known to used by this actor.

        Returns:
            [list[Malware]] -- A list of malware objects defined within the
                      Enterprise MITRE ATT&CK Framework
        """
        return self._get_relationship_objects(
            parent_id=self.id,
            parent_type='malware'
        )
 
    @property
    def tools(self):
        """
        Returns all tool objects that are known to used by this actor.

        Returns:
            [list[Tool]] -- A list of tool objects defined within the
                      Enterprise MITRE ATT&CK Framework
        """
        return self._get_relationship_objects(
            parent_id=self.id,
            parent_type='tool'
        )

    @property
    def techniques(self):
        """
        Returns all technique objects that this actor is known to use.

        Returns:
            [list[Technique]] -- A list of technique objects defined within the
                      Enterprise MITRE ATT&CK Framework
        """
        return self._get_relationship_objects(
            parent_id=self.id,
            parent_type='attack-pattern'
        )



import re
from pydantic import BaseModel


# https://ihateregex.io/expr/semver/

PATTERNS = {
    'semversion': {
        'pattern': "^(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)(?:-((?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\+([0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$",
        'examples': ['1.1.1', '0.1.2', '99.99.99']
    },
    'types': {
        'pattern': None,
        'examples': ['relationship', 'x-mitre-matrix', 'identity', 'marking-definition', 'course-of-action', 'malware', 'tool', 'intrusion-set', 'x-mitre-data-source', 'x-mitre-data-component', 'x-mitre-tactic', 'attack-pattern', 'bundle']
    },
    'reference': {
        'pattern': None,
        'examples': ['identity', 'marking-definition', 'course-of-action', 'malware', 'tool', 'intrusion-set', 'x-mitre-data-source', 'x-mitre-data-component', 'x-mitre-tactic', 'attack-pattern']
    },
    'domains': {
        'pattern': None,
        'examples': ['mobile-attack', 'enterprise-attack']
    },
    'platforms': {
        'pattern': None,
        'examples': ['Windows', 'Android', 'iOS', 'macOS', 'Azure AD', 'SaaS', 'Network', 'Google Workspace', 'PRE', 'Containers', 'IaaS', 'Linux', 'Office 365']
    }
}


REGEXS = {
    'semversion': re.compile(PATTERNS['semversion']['pattern'])
}


class BaseCustomType(str):

    @classmethod
    def __get_validators__(cls):
        # one or more validators may be yielded which will be called in the
        # order to validate the input, each validator will receive as an input
        # the value returned from the previous validator
        yield cls.validate


class SemVersion(BaseCustomType):

    @classmethod
    def __modify_schema__(cls, field_schema):
        # __modify_schema__ should mutate the dict it receives in place,
        # the returned value will be ignored
        field_schema.update(
            # simplified regex here for brevity, see the wikipedia link above
            pattern=PATTERNS['semversion']['pattern'],
            # some example postcodes
            examples=PATTERNS['semversion']['examples'],
        )

    @classmethod
    def validate(cls, v):
        if not isinstance(v, str):
            raise TypeError('string required')
        m = REGEXS['semversion'].fullmatch(v.upper())
        if not m:
            raise ValueError('invalid sem version format')
        return cls(f'{m.group(1)} {m.group(2)}')

    def __repr__(self):
        return f'SemVersion({super().__repr__()})'


class Id(BaseCustomType):

    @classmethod
    def __modify_schema__(cls, field_schema):
        # __modify_schema__ should mutate the dict it receives in place,
        # the returned value will be ignored
        field_schema.update(
            examples=PATTERNS['types']['examples'],
        )

    @classmethod
    def validate(cls, v):
        if not isinstance(v, str):
            raise TypeError('string required')
        type,id = v.split('--')
        if type not in PATTERNS['types']['examples']:
            raise ValueError('Invalid Id attribute.')
        return cls(v)

    def __repr__(self):
        return f'Id({super().__repr__()})'


class MitreDomain(BaseCustomType):

    @classmethod
    def __modify_schema__(cls, field_schema):
        # __modify_schema__ should mutate the dict it receives in place,
        # the returned value will be ignored
        field_schema.update(
            examples=PATTERNS['domains']['examples'],
        )

    @classmethod
    def validate(cls, v):
        if not isinstance(v, str):
            raise TypeError('string required')
        type,id = v.split('--')
        if type not in PATTERNS['domains']['examples']:
            raise ValueError('Invalid MitreDomain attribute.')
        return cls(v)

    def __repr__(self):
        return f'MitreDomain({super().__repr__()})'


class MitrePlatform(BaseCustomType):

    @classmethod
    def __modify_schema__(cls, field_schema):
        # __modify_schema__ should mutate the dict it receives in place,
        # the returned value will be ignored
        field_schema.update(
            examples=PATTERNS['platforms']['examples'],
        )

    @classmethod
    def validate(cls, v):
        if not isinstance(v, str):
            raise TypeError('string required')
        type,id = v.split('--')
        if type not in PATTERNS['platforms']['examples']:
            raise ValueError('Invalid MitrePlatform attribute.')
        return cls(v)

    def __repr__(self):
        return f'MitrePlatform({super().__repr__()})'




from datetime import datetime
from typing import List, AnyStr

from attrs import define, field
from pydantic import HttpUrl


@define
class Query:
    product: AnyStr = field()
    query: AnyStr = field()
    name: AnyStr = field()


@define
class TechniqueCommand:
    command: AnyStr = field()
    source: HttpUrl = field()
    name: AnyStr = field(factory=str)


@define
class Technique:
    technique_id: AnyStr = field()
    command_list: list = field(factory=list)
    commands: List[TechniqueCommand] = field(factory=list)
    parsed_datasets: list = field(factory=list)
    queries: List[Query] = field(factory=list)
    possible_detections: list = field(factory=list)
    external_reference: list = field(factory=list)


@define
class C2Data:
    name: AnyStr = field(factory=str)
    license: AnyStr = field(factory=str)
    price: AnyStr = field(factory=str)
    github: AnyStr = field(factory=str)
    site: AnyStr = field(factory=str)
    twitter: AnyStr = field(factory=str)
    evaluator: AnyStr = field(factory=str)
    date: AnyStr = field(factory=str)
    version: int = field(factory=int)
    implementation: AnyStr = field(factory=str)
    how_to: AnyStr = field(factory=str)
    slingshot: AnyStr = field(factory=str)
    kali: AnyStr = field(factory=str)
    server: AnyStr = field(factory=str)
    implant: AnyStr = field(factory=str)
    multi_user: bool = field(factory=bool)
    ui: bool = field(factory=bool)
    dark_mode: bool = field(factory=bool)
    api: bool = field(factory=bool)
    windows: bool = field(factory=bool)
    linux: bool = field(factory=bool)
    macos: bool = field(factory=bool)
    tcp: bool = field(factory=bool)
    http: bool = field(factory=bool)
    http2: bool = field(factory=bool)
    http3: bool = field(factory=bool)
    dns: bool = field(factory=bool)
    doh: bool = field(factory=bool)
    icmp: bool = field(factory=bool)
    ftp: bool = field(factory=bool)
    imap: bool = field(factory=bool)
    mapi: bool = field(factory=bool)
    smb: bool = field(factory=bool)
    ldap: bool = field(factory=bool)
    key_exchange: bool = field(factory=bool)
    stego: bool = field(factory=bool)
    proxy_aware: bool = field(factory=bool)
    domainfront: bool = field(factory=bool)
    custom_profile: bool = field(factory=bool)
    jitter: bool = field(factory=bool)
    working_hours: bool = field(factory=bool)
    kill_date: bool = field(factory=bool)
    chaining: bool = field(factory=bool)
    logging: bool = field(factory=bool)
    in_wild: bool = field(factory=bool)
    attck_mapping: bool = field(factory=bool)
    dashboard: bool = field(factory=bool)
    blog: AnyStr = field(factory=str)
    c2_matrix_indicators: AnyStr = field(factory=str)
    jarm: bool = field(factory=bool)
    actively_maint: bool = field(factory=bool)
    slack: bool = field(factory=bool)
    slack_members: bool = field(factory=bool)
    gh_issues: bool = field(factory=bool)
    notes: AnyStr = field(factory=str)


@define
class Tool:
    names: list = field()
    comments: AnyStr = field()
    family: list = field(factory=list)
    links: list = field(factory=list)


@define
class Actor:
    country: AnyStr = field()
    names: List = field()
    targets: AnyStr = field()
    operations: List = field()
    description: AnyStr = field()
    tools: List = field()
    links: List = field()
    attck_id: AnyStr = field()
    comment: AnyStr = field()


@define
class GeneratedData:
    last_updated: datetime = field()
    techniques: List[Technique] = field(factory=list)
    c2_data: List[C2Data] = field(factory=list)
    tools: List[Tool] = field(factory=list)
    actors: List[Actor] = field(factory=list)

    @last_updated.default
    def default_updated(self):
        return str(datetime.now())

    def add_actor_item(self, country, names, targets, operations, description, tools, links, attack_id, comment):
        self.actors.append(
            Actor(
                country=country,
                names=list(set(names)),
                targets=list(set(targets)),
                operations=list(set(operations)),
                description=description,
                tools=list(set(tools)),
                links=list(set(links)),
                attck_id=attack_id,
                comment=comment
            )
        )

    def add_tool_item(self, names, comments, family, links):
        tool = Tool(
            names=list(set(names)),
            comments=comments,
            family=family,
            links=list(set(links))
        )
        if self.tools:
            if tool not in self.tools:
                self.tools.append(tool)
        else:
            self.tools.append(tool)

    def add_command(self, technique_id, source, name, command):
        c = TechniqueCommand(
            command=command,
            source=source,
            name=name
        )
        if self.techniques:
            found = False
            for technique in self.techniques:
                if technique.technique_id == technique_id:
                    found = True
                    if technique.commands:
                        if c not in technique.commands:
                            technique.commands.append(c)
                    else:
                        technique.commands.append(c)

                    if technique.command_list:
                        if command not in technique.command_list:
                            technique.command_list.append(command)
                    else:
                        technique.command_list.append(command)
            if not found:
                self.techniques.append(
                    Technique(
                        technique_id=technique_id,
                        commands=[c],
                        command_list=[command]
                    )
                )
        else:
            self.techniques.append(
                    Technique(
                        technique_id=technique_id,
                        commands=[c],
                        command_list=[command]
                    )
                )

    def add_dataset(self, technique_id, content):
        if self.techniques:
            found = False
            for technique in self.techniques:
                if technique.technique_id == technique_id:
                    found = True
                    technique.parsed_datasets.append(content)
            if not found:
                self.techniques.append(
                    Technique(
                        technique_id=technique_id,
                        parsed_datasets=[content]
                    )
                )
        else:
            self.techniques.append(
                    Technique(
                        technique_id=technique_id,
                        parsed_datasets=[content]
                    )
                )

    def add_possible_queries(self, technique_id, product, content, name):
        q = Query(
            product=product,
            query=content,
            name=name
        )
        if self.techniques:
            found = False
            for technique in self.techniques:
                if technique.technique_id == technique_id:
                    found = True
                    technique.queries.append(q)
            if not found:
                self.techniques.append(
                    Technique(
                        technique_id=technique_id,
                        queries=[q]
                    )
                )
        else:
            self.techniques.append(
                    Technique(
                        technique_id=technique_id,
                        queries=[q]
                    )
                )

    def add_possible_detection(self, technique_id, data):
        if self.techniques:
            found = False
            for technique in self.techniques:
                if technique.technique_id == technique_id:
                    found = True
                    technique.possible_detections.append(data)
            if not found:
                self.techniques.append(
                    Technique(
                        technique_id=technique_id,
                        possible_detections=[data]
                    )
                )
        else:
            self.techniques.append(
                    Technique(
                        technique_id=technique_id,
                        possible_detections=[data]
                    )
                )

    def add_external_reference(self, technique_id, reference):
        if self.techniques:
            found = False
            for technique in self.techniques:
                if technique.technique_id == technique_id:
                    found = True
                    if reference not in technique.external_reference:
                        technique.external_reference.append(reference)
            if not found:
                self.techniques.append(
                    Technique(
                        technique_id=technique_id,
                        external_reference=[reference]
                    )
                )
        else:
            self.techniques.append(
                    Technique(
                        technique_id=technique_id,
                        external_reference=[reference]
                    )
                )

    def add_c2_data(self, data):
        if isinstance(data, dict):
            new_dict = {}
            for key, val in data.items():
                new_dict[key.replace(' ','_').replace('-','_').lower()] = val
            data = C2Data(**new_dict)
            if data not in self.c2_data:
                self.c2_data.append(data)



from .types import (
    Id, 
    SemVersion,
    MitreDomain,
    MitrePlatform
)
from .base import (
    BaseModel,
    List,
    AnyStr,
    define,
    field,
    validators
)


@define
class Malware(BaseModel):
    type: AnyStr = field(validator=validators.in_(['malware']))
    labels: List = field()
    description: AnyStr = field()
    x_mitre_attack_spec_version: SemVersion = field()
    x_mitre_modified_by_ref: Id = field()
    x_mitre_aliases: List = field()
    x_mitre_platforms: List[MitrePlatform] = field()
    x_mitre_contributors: List = field()
    x_mitre_domains: List[MitreDomain] = field()

    revoked: bool = field(factory=bool)
    created_by_ref: Id = field(factory=Id)
    x_mitre_deprecated: bool = field(factory=bool)

    @property
    def actors(self):
        return self._get_relationship_objects(
            parent_id=self.id,
            parent_type='intrusion-set'
        )

    @property
    def techniques(self):
        return self._get_relationship_objects(
            parent_id=self.id,
            parent_type='attack-pattern'
        )



from .types import (
    Id, 
    SemVersion,
    MitreDomain
)
from .base import (
    BaseModel,
    List,
    AnyStr,
    define,
    field,
    validators
)


@define
class Matrix(BaseModel):
    type: AnyStr = field(validator=validators.in_(['x-mitre-matrix']))
    tactic_refs: List[Id] = field()
    created_by_ref: Id = field()
    description: AnyStr = field()
    x_mitre_attack_spec_version: SemVersion = field()
    x_mitre_modified_by_ref: Id = field()
    x_mitre_domains: List[MitreDomain] = field(factory=list)


from datetime import datetime
from attr import validators
from .types import (
    Id
)
from .base import (
    ExternalReferences,
    List,
    AnyStr,
    define,
    field
)


@define
class Identity:
    id: Id = field()
    type: AnyStr = field(validator=validators.in_(['identity']))
    identity_class: AnyStr = field()
    created: datetime = field()
    modified: datetime = field()
    name: AnyStr = field()
    external_references: List[ExternalReferences] = field()
    object_marking_refs: List[Id] = field()
    roles: List = field(factory=list)
    sectors: List = field(factory=list)



from datetime import datetime

from attrs import validators

from .types import Id
from .base import (
    List,
    AnyStr,
    define,
    field,
    ExternalReferences
)


@define
class ControlObject:
    id: Id = field()
    name: AnyStr = field()
    created: datetime = field()
    external_references: List[ExternalReferences] = field()
    modified: datetime = field()
    description: AnyStr = field()
    type: AnyStr = field(validator=validators.in_(['course-of-action']))
    x_mitre_family: AnyStr = field()
    x_mitre_priority: AnyStr = field()
    x_mitre_impact: list = field(factory=list)


@define
class NistControls:
    id: Id = field()
    objects: List[ControlObject] = field(factory=list)

    def __attrs_post_init__(self):
        if self.objects:
            return_list = []
            for item in self.objects:
                try:
                    return_list.append(ControlObject(**item))
                except Exception as e:
                    raise e
            self.objects = return_list



"""Command-line interface."""
import click


@click.command()
@click.version_option()
def main() -> None:
    """Pyattck Data Models."""


if __name__ == "__main__":
    main(prog_name="pyattck-data-models")  # pragma: no cover



from datetime import datetime
from attr import validators
from .types import (
    Id,
    SemVersion
)
from .base import (
    ExternalReferences,
    List,
    AnyStr,
    define,
    field
)


@define
class Relationship:
    id: Id = field()
    type: AnyStr = field(validator=validators.in_(['relationship']))
    created: datetime = field()
    external_references: List[ExternalReferences] = field()
    modified: datetime = field()
    object_marking_refs: List[Id] = field()
    relationship_type: AnyStr = field()
    source_ref: Id = field()
    target_ref: Id = field()
    x_mitre_modified_by_ref: Id = field()
    revoked: bool = field(factory=bool)
    created_by_ref: Id = field(factory=Id)
    description: AnyStr = field(factory=str)
    x_mitre_deprecated: bool = field(factory=bool)
    x_mitre_version: SemVersion = field(factory=SemVersion)
    x_mitre_attack_spec_version: SemVersion = field(factory=SemVersion)



from .base import (
    BaseModel,
    List,
    AnyStr,
    define,
    field
)


@define
class Control(BaseModel):
    revoked: bool = field()
    x_mitre_family: AnyStr = field(factory=AnyStr)
    x_mitre_impact: List = field(factory=list)
    x_mitre_priority: AnyStr = field(factory=AnyStr)

    @property
    def techniques(self):
        return self._get_relationship_objects(
            parent_id=self.id,
            parent_type='attack-pattern'
        )



from .base import (
    BaseModel,
    List,
    AnyStr,
    define,
    field,
    validators
)
from .types import (
    Id,
    SemVersion,
    MitreDomain
)


@define
class DataComponent(BaseModel):
    type: AnyStr = field(validator=validators.in_(['x-mitre-data-component']))
    description: AnyStr = field()
    created_by_ref: Id = field()
    x_mitre_modified_by_ref: Id = field()
    x_mitre_data_source_ref: Id = field()
    object_marking_refs: List[Id] = field()
    x_mitre_domains: List[MitreDomain] = field()
    x_mitre_attack_spec_version: SemVersion = field()

    @property
    def techniques(self):
        return self._get_relationship_objects(
            parent_id=self.id,
            parent_type='attack-pattern'
        )
